/*
 *@(#)HelloMIDlet.java	1.3 00/05/22
 * Copyright (c) 2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;
import java.io.*;

import org.kxml.*;
import org.kxml.parser.*;

/**
 * An example MIDlet with simple "Hello" text and an Exit command.
 * Refer to the startApp, pauseApp, and destroyApp
 * methods so see how each handles the requested transition.
 */
public class HelloMIDlet extends MIDlet implements CommandListener {

    private Command exitCommand; // The exit command
	 private Command xmlGetCommand;
	 private Command xmlPostCommand;
    private Display display;	// The display for this MIDlet
	 private final static String SERVLET_URL =
		  "http://127.0.0.1:8080/ServletXML/ServletXML";
	 private static TextBox t;
	 private static String textBoxString = "";

    public HelloMIDlet() {
        display = Display.getDisplay(this);
	     exitCommand = new Command("Exit", Command.EXIT, 2);
		  xmlGetCommand  = new Command( "Get", Command.SCREEN, 1 );
		  xmlPostCommand  = new Command( "Post", Command.SCREEN, 1 );
    }

    /**
     * Start up the Hello MIDlet by creating the TextBox and associating
     * the exit command and listener.
     */
    public void startApp() {
        t = new TextBox("MIDlet XML", "Using kXML", 256, 0);

        t.addCommand(exitCommand);
		  t.addCommand( xmlGetCommand );
		  t.addCommand( xmlPostCommand );
        t.setCommandListener(this);

        display.setCurrent(t);
    }

    /**
     * Pause is a no-op since there are no background activities or
     * record stores that need to be closed.
     */
    public void pauseApp() {
    }

    /**
     * Destroy must cleanup everything not handled by the garbage collector.
     * In this case there is nothing to cleanup.
     */
    public void destroyApp(boolean unconditional) {
    }

    /**
     * Traverses the XML file
     */
    
	 public static void traverse (XmlParser parser, String indent) 
			throws IOException {
    
	 boolean leave = false;
	 String st = new String();
  
	  do {
	    ParseEvent event = parser.read ();

	    switch (event.getType ()) {
	    case Xml.START_TAG:

		// see API doc of StartTag for more access methods

		st = indent + "start tag: '"+event.getName () +
				"'; attributes: "+event.getAttributes ();
		System.out.println( st );
		textBoxString += st;
		traverse (parser, indent + "."); // recursion
		break;
		
		 case Xml.END_TAG:

			st = indent + "/end tag: '"+event.getName ()+"'";
			System.out.println (st);
			textBoxString += st;
			leave = true;
		break;

	    case Xml.END_DOCUMENT:
		
			st = indent + "Document end reached!";
		System.out.println (st);
		textBoxString += st;
		leave = true;
		break;
		
	    case Xml.TEXT:
		
			st = indent + "text: "+event.getText ();
			textBoxString += st;
		System.out.println (st);
		break;

	    case Xml.WHITESPACE:
		
		System.out.println (indent + "ignoring whitespace");
		break;

	    default:
		System.out.println 
		    (indent + "ignoring some other (legacy) event: "+event);
	    }
	}
	while (!leave);
		t.setString( textBoxString );
    }


    /*
     * Respond to commands, including exit
     * On the exit command, cleanup and notify that the MIDlet has been destroyed.
     */
   public void commandAction(Command c, Displayable s) {
	if (c == exitCommand) {
	    destroyApp(false);
	    notifyDestroyed();
	}	
	else if ( c == xmlGetCommand ) {
		try
		{
			String xmlString = getXML();
			byte[] xmlByteArray = xmlString.getBytes();
			ByteArrayInputStream xmlStream = new ByteArrayInputStream(xmlByteArray);
			InputStreamReader xmlReader = new InputStreamReader(xmlStream);
			XmlParser parser = new XmlParser(xmlReader);
			traverse( parser, "" );
		}
		catch( Exception e )
		{
			e.printStackTrace();
		}

	}
	else if ( c == xmlPostCommand ) {
		try {
			doPost();
		}
		catch( Exception e ) {
			e.printStackTrace();
		}
	}

	}
    

	 public String getXML() throws IOException
	 {
		 HttpConnection conn = null;
		 InputStream servletStream = null;
		 OutputStream os = null;
		 StringBuffer xmlBuffer = new StringBuffer();
		 String xmlString = null;
	
		 try {
			conn = (HttpConnection)Connector.open(SERVLET_URL);
			System.out.println( "Opened a HttpConnection to: " + SERVLET_URL );
	
			/*  Send the parameters to the servlet via GET method  */
			conn.setRequestMethod(HttpConnection.GET );
			os = conn.openOutputStream();
			os.flush();
	
			servletStream = conn.openDataInputStream();
			int ch;
	
			while ((ch = servletStream.read()) != -1)
			  xmlBuffer.append((char)ch);
	
			xmlString = xmlBuffer.toString();
			
			/*  Ensure that we tidy everything up  */
			if (servletStream != null)
			  servletStream.close();
	 
			if (conn != null)
			  conn.close();
	
			if (os != null)
			  os.close();
			
			 System.out.println( "Instructions are: " + xmlString );
			 return xmlString;

		 } catch (IOException e) {
			System.out.println("IO error getting xmlString from server");
			/*  Ensure that we tidy everything up  */
			if (servletStream != null)
			  servletStream.close();
	 
			if (conn != null)
			  conn.close();
	
			if (os != null)
			  os.close();
			return null;
		 } finally {
			 System.out.println( "In finally..." );
			//  Ensure that we tidy everything up  
			if (servletStream != null)
			  servletStream.close();
	 
			if (conn != null)
			  conn.close();
	
			if (os != null)
			  os.close(); 
		   return xmlString;
	 }
  }

  public void doPost() throws IOException {
	  InputStream is = null;
	  OutputStream os = null;
	  HttpConnection conn = null;
	  try {
		  StringBuffer buffer = new StringBuffer();

		  conn = (HttpConnection)Connector.open(SERVLET_URL);
		  System.out.println( "Opened a HttpConnection to: " + SERVLET_URL );
  
		  /*  Send the parameters to the servlet via POST method  */
		  conn.setRequestMethod( HttpConnection.POST );
		  conn.setRequestProperty( "ticker", "SUNW" );
		  conn.setRequestProperty( "price", "20" );
		  conn.setRequestProperty( "Company", "Sun Microsystems" );
		  os = conn.openOutputStream();
		  os.flush();
  
		  is = conn.openDataInputStream();
		  int ch;
  
		  while ((ch = is.read()) != -1)
			 buffer.append((char)ch);
  
		  String resultString = buffer.toString();
		  
		  /*  Ensure that we tidy everything up  */
		  if (is != null) is.close();
	
		  if (conn != null) conn.close();
  
		  if (os != null) os.close();
		  
			System.out.println( "results are: " + resultString );

	  }
	  catch( IOException e ) {
		  e.printStackTrace();
		  if (conn != null) conn.close();
		  if (os != null)   os.close();
		  if ( is != null ) is.close(); 
	  }
  }


}
